@@ -33,6 +33,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <errno.h>
 
 #include <rfid_reader.h>
 #include <s6350.h>
@@ -63,8 +64,8 @@ static int error_map[256] = {
   /* 0x0e */ ENOSYS, 
   /* 0x0f */ EDOM, /* "Error with no information given or a specific error code is not supported" */
   /* 0x10 */ ENODEV, /* "The specified block is not available (doesn't exist)" */
-  /* 0x11 */ ENOLCK, /* "The specified block is already locked and thus cannot be locked again" */
-  /* 0x12 */ EPERM, /* "The specified block is locked and its content cannot be changed" */
+  /* 0x11 */ EPERM, /* "The specified block is already locked and thus cannot be locked again" */
+  /* 0x12 */ ENOLCK, /* "The specified block is locked and its content cannot be changed" */
   /* 0x13 */ ENOMEM, /* "The specified block was not successfully programmed" */
   /* 0x14 */ ESRCH, /* "The specified block was not successfully locked" */
   /* 0x15 */ ENOENT, /* not ISO-15693 "Transponder not found" */
@@ -80,6 +81,29 @@ static int error_map[256] = {
   /* 0x1f */ ENOSYS,
 };
 
+/*
+ * Return 0 if the transponders are equal, -1 if a < b, 1 if a > b.
+ * Comparison is made on the UID of each transponder.
+ */
+static int iso15693_transponder_cmp(iso15693_transponder_t* a, iso15693_transponder_t* b)
+{
+  return memcmp(a->uid, b->uid, ISO15693_UID_SIZE);
+}
+
+/*
+ * Return true if the UID of the transponder is all 0, return false otherwise.
+ */
+static int iso15693_transponder_null(iso15693_transponder_t* transponder)
+{
+  int i;
+
+  for(i = 0; i < ISO15693_UID_SIZE; i++)
+    if(transponder->uid[i])
+      return 0;
+
+  return 1;
+}
+
 static u_int16_t crc16_compute(u_int8_t* buffer, int buffer_size)
 {
   /* NOT IMPLEMENTED */
@@ -312,7 +336,7 @@ int iso15693_read_single_block(iso15693_t* iso15693, rfid_transponder_t* transpo
     index += ISO15693_UID_SIZE;
   }
 
-  if(block->block_number > transponder->blocks) {
+  if(block->block_number >= transponder->blocks) {
     iso15693->error = E2BIG;
     return -1;
   }
@@ -367,7 +391,7 @@ int iso15693_write_single_block(iso15693_t* iso15693, const rfid_transponder_t*
     index += ISO15693_UID_SIZE;
   }
 
-  if(block->block_number > transponder->blocks) {
+  if(block->block_number >= transponder->blocks) {
     iso15693->error = E2BIG;
     return -1;
   }
@@ -414,7 +438,7 @@ int iso15693_read_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_t
     index += ISO15693_UID_SIZE;
   }
 
-  if(block_start > transponder->blocks ||
+  if(block_start >= transponder->blocks ||
      block_start + blocks_length > transponder->blocks) {
     iso15693->error = E2BIG;
     return -1;
@@ -480,7 +504,7 @@ int iso15693_write_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_
     index += ISO15693_UID_SIZE;
   }
 
-  if(block_start > transponder->blocks ||
+  if(block_start >= transponder->blocks ||
      block_start + blocks_length > transponder->blocks) {
     iso15693->error = E2BIG;
     free(request);
@@ -514,7 +538,7 @@ int iso15693_write_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_
   return 0;
 }
 
-int iso15693_lock_blocks(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* block)
+int iso15693_lock_block(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* block)
 {
   u_int8_t request[ISO15693_UID_SIZE + ISO15693_BLOCK_NUMBER_SIZE];
   int request_length;
@@ -693,22 +717,6 @@ int iso15693_get_system_information(iso15693_t* iso15693, rfid_transponder_t* tr
   return 0;
 }
 
-int iso15693_transponder_cmp(iso15693_transponder_t* a, iso15693_transponder_t* b)
-{
-  return memcmp(a->uid, b->uid, ISO15693_UID_SIZE);
-}
-
-int iso15693_transponder_null(iso15693_transponder_t* transponder)
-{
-  int i;
-
-  for(i = 0; i < ISO15693_UID_SIZE; i++)
-    if(transponder->uid[i])
-      return 0;
-
-  return 1;
-}
-
 static const char* drv_version(struct rfid_reader* reader)
 {
   return RFID_VERSION;
@@ -731,7 +739,7 @@ static int drv_init(struct rfid_reader* reader)
   if(s6350_reader_alloc(&iso15693->reader) < 0)
     return -1;
 
-  iso15693->reader->parent = reader;
+  iso15693->reader->iso15693_reader = reader;
   iso15693->reader->device = iso15693->device;
 
   if(iso15693->reader->init_f(iso15693->reader) < 0)
@@ -786,13 +794,15 @@ static int drv_read(struct rfid_reader* reader, int block_start, int number_of_b
   int i;
   u_int8_t* pointer;
 
-  if(block_start > transponder->blocks ||
+  if(block_start >= transponder->blocks ||
      block_start + number_of_blocks > transponder->blocks) {
     iso15693->error = E2BIG;
     return -1;
   }
 
-  transponder->data_length = number_of_blocks * transponder->bytes_per_block;
+  transponder->data_length = transponder->blocks * transponder->bytes_per_block;
+  transponder->data[transponder->data_length] = '\0'; /* for trailing NULL */
+
   pointer = transponder->data + block_start * transponder->bytes_per_block;
   
   for(i = 0; i < number_of_blocks; i++) {
@@ -1027,7 +1037,7 @@ static int drv_transponder_copy(struct rfid_reader* reader, rfid_transponder_t*
   return 0;
 }
 
-static int drv_transponder_id_get(struct rfid_reader* reader, rfid_transponder_t* transponder, char** idp)
+static int drv_transponder_id_get(struct rfid_reader* reader, const rfid_transponder_t* transponder, char** idp)
 {
   iso15693_transponder_t* iso15693_transponder = (iso15693_transponder_t*)transponder->private;
 
