@@ -24,6 +24,10 @@
  *
  */
 
+/*
+ * For a detailed description see iso15693(3)
+ */
+
 #ifndef _ISO15693_H_
 #define _ISO15693_H_
 
@@ -67,18 +71,17 @@
  * for instance). If the implementation of ISO-15693 proposed by the
  * RFID reader is so different that hiding the differences requires a
  * complex mechanism, it is best to call it a proprietary protocol and
- * implement it in a reader of its own.
+ * not attempt to pretend it's ISO-15693.
  */
 
-typedef struct {
+typedef struct iso15693 {
   u_int8_t flags;		/* Permanent flags (see
-				   ISO15693_COMMAND_FLAGS_* */
-  rfid_reader_t* reader;	/* RFID reader handler. */
-  int error;			/* Errors set by rfid functions. */
+				   ISO15693_COMMAND_FLAGS_*) */
+  rfid_reader_t* reader;	/* RFID reader handler, the actual device. */
+  int error;			/* Errors set by iso15693 functions. */
   char* device;			/* Path name of the device connecting
 				   to the RFID reader. */
-  int verbose;			/* Control verbosity level of
-				   functions. */
+  int verbose;			/* Control verbosity level. */
 } iso15693_t;
 
 /*
@@ -191,7 +194,7 @@ typedef struct {
 #define ISO15693_CRC_SIZE			2
 #define ISO15693_AFI_SIZE			1
 #define ISO15693_MASK_LENGTH_SIZE		1
-#define ISO15693_MASK_VALUE_SIZE		8 /* Maximum size in bits */
+#define ISO15693_MASK_VALUE_SIZE		8
 #define ISO15693_UID_SIZE			8
 #define ISO15693_DSFID_SIZE			1
 #define ISO15693_BLOCK_NUMBER_SIZE		1
@@ -247,14 +250,18 @@ typedef struct {
 #define ISO15693_INFO_FLAGS_VICC_MEMORY_SIZE	0x04
 #define ISO15693_INFO_FLAGS_IC_REFERENCE	0x08
 
+/*
+ * Meta information about the RFID transponder. (ro) means
+ * readonly, (rw) means read-write.
+ */
 typedef struct {
   u_int8_t info_flags;          /* Meaning is the same as in
 				   GET_SYSTEM_INFORMATION command. */
-  u_int8_t uid[ISO15693_UID_SIZE];
-  u_int8_t dsfid;
-  u_int8_t afi;
-  u_int16_t vicc_memory;
-  u_int8_t ic_ref;
+  u_int8_t uid[ISO15693_UID_SIZE]; /* Unique identifier (ro). */
+  u_int8_t dsfid;		/* Data storage format identifier (rw). */
+  u_int8_t afi;			/* Application family identifier (rw). */
+  u_int16_t vicc_memory;	/* Blocks layout information (ro). */
+  u_int8_t ic_ref;		/* Proprietary information (ro). */
 } iso15693_transponder_t;
 
 /*
@@ -549,6 +556,41 @@ typedef struct {
 #define ISO15693_COMMAND_INVENTORY				0x01
 
 /*
+ * Input parameter of the inventory command.
+ */
+typedef struct {
+  u_int8_t mask_length;				 /* Number of valid BITS in 
+						    mask_value. If 0, it is ignored. 
+						 */
+  u_int8_t mask_value[ISO15693_MASK_VALUE_SIZE]; /* Mask used for inventory pattern
+						    matching algorithm. See ISO-15693
+						    for a description of the matching
+						    algorithm. */
+  u_int8_t afi;					 /* Mask used for the inventory 
+						    pattern matching algorithm. See
+						    AFI Application family identifier
+						    above for a table explaining
+						    how it is used. If 0, it is
+						    ignored. */
+} iso15693_inventory_t;
+
+/*
+ * Output result of the inventory command (an array of these is actually
+ * returned).
+ */
+typedef struct {
+  u_int8_t dsfid;			/* Data Storage Format Identifier. */
+  u_int8_t uid[ISO15693_UID_SIZE];	/* Unique identifier. */
+} iso15693_inventory_result_t;
+
+/*
+ * The INVENTORY command is sent using the <inventory> parameters. If
+ * no transponder is found *resultsp and *results_lengthp are set to 0.
+ * If at least one transponder is found, an array of iso15693_inventory_result_t
+ * structures is malloc'ed and returned in *resultsp. The number of transponders
+ * is set to *results_lengthp. It is the responsibility of the caller
+ * to free the *resultsp array.
+ *
  * Since ISO-15693 is NOT a host <-> reader protocol but a reader <->
  * transponder protocol, the INVENTORY command cannot be used as-is by
  * readers that use ISO-15693 as a host <-> reader protocol. An INVENTORY
@@ -556,17 +598,6 @@ typedef struct {
  * descriptions, matching the transponders currently detected by the
  * reader.
  */
-typedef struct {
-  u_int8_t mask_length;
-  u_int8_t mask_value[ISO15693_MASK_VALUE_SIZE];
-  u_int8_t afi;
-} iso15693_inventory_t;
-
-typedef struct {
-  u_int8_t dsfid;
-  u_int8_t uid[ISO15693_UID_SIZE];
-} iso15693_inventory_result_t;
-
 int iso15693_inventory(iso15693_t* iso15693, iso15693_inventory_t* inventory, iso15693_inventory_result_t** resultsp, int* results_lengthp);
 
 /*
@@ -575,7 +606,7 @@ int iso15693_inventory(iso15693_t* iso15693, iso15693_inventory_t* inventory, is
  * When receiving the Stay quiet command, the VICC shall enter the
  * quiet state and shall NOT send back a response. There is NO
  * response to the Stay quiet command. When in quiet state: a) the VICC
- * shall not process any request where Inventory_flag is set, b) the
+ * shall not process any request where inventory flag is set, b) the
  * VICC shall process any addressed request. The VICC shall exit the
  * quiet state when: a) reset (power off), b) receiving a Select
  * request (it shall then go to the selected state if supported or
@@ -592,6 +623,11 @@ int iso15693_inventory(iso15693_t* iso15693, iso15693_inventory_t* inventory, is
  */
 #define ISO15693_COMMAND_STAY_QUIET				0x02
 
+/*
+ * The STAY_QUIET command is sent to the transponder with 
+ * UID equal to <uid> (ISO15693_UID_SIZE bytes).
+ * 
+ */
 int iso15693_stay_quiet(iso15693_t* iso15693, u_int8_t* uid);
 
 /*
@@ -619,6 +655,21 @@ int iso15693_stay_quiet(iso15693_t* iso15693, u_int8_t* uid);
  */
 #define ISO15693_COMMAND_READ_SINGLE_BLOCK			0x20
 
+/*
+ * The READ_SINGLE_BLOCK is sent to the first available transponder if
+ * the UID is not set in <transponder> (iso15693_transponder_null) or
+ * to a designated transponder. Meta information of <transponder> must
+ * be set (using GET_SYSTEM_INFORMATION for instance) before calling
+ * this function. The requested block number is <block->block_number>. If
+ * <block->security_status> is not 0, the block security status will
+ * also be requested. On success, the <block->data> area is filled
+ * with the content of block number <block->block_number>. If the block
+ * security status was requested, the <block->security_status> is set
+ * with its value. If it was not requested, the contend of
+ * <block->security_status> is undefined.  It is the responsibility of
+ * the caller to set <block->data> to a pointer with enough room to
+ * store the content of a block.
+ */
 int iso15693_read_single_block(iso15693_t* iso15693, rfid_transponder_t* transponder, rfid_block_t* block);
 
 /*
@@ -648,6 +699,15 @@ int iso15693_read_single_block(iso15693_t* iso15693, rfid_transponder_t* transpo
  */
 #define ISO15693_COMMAND_WRITE_SINGLE_BLOCK			0x21
 
+/*
+ * The WRITE_SINGLE_BLOCK is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. Meta information of <transponder>
+ * must be set (using GET_SYSTEM_INFORMATION for instance) before
+ * calling this function. On success, <transponder->bytes_per_block> bytes
+ * are copied from <block->data> to the <block->block_number> block of
+ * the transponder.
+ */
 int iso15693_write_single_block(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* block);
 
 /*
@@ -676,7 +736,13 @@ int iso15693_write_single_block(iso15693_t* iso15693, const rfid_transponder_t*
  */
 #define ISO15693_COMMAND_LOCK_BLOCK				0x22
 
-int iso15693_lock_blocks(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* block);
+/*
+ * The LOCK_BLOCK is sent to the first available transponder if the
+ * UID is not set in <transponder> (iso15693_transponder_null) or to a
+ * designated transponder. On success the <block->block_number> block
+ * is locked and becomes readonly.
+ */
+int iso15693_lock_block(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* block);
 
 /*
  * READ_MULTIPLE_BLOCKS:
@@ -710,6 +776,19 @@ int iso15693_lock_blocks(iso15693_t* iso15693, const rfid_transponder_t* transpo
  */
 #define ISO15693_COMMAND_READ_MULTIPLE_BLOCKS			0x23
 
+/*
+ * The READ_MULTIPLE_BLOCK is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. Meta information of <transponder>
+ * must be set (using GET_SYSTEM_INFORMATION for instance) before
+ * calling this function. On success, the <blocks[i]->data> area of
+ * each <blocks> is filled with the content of the consecutive blocks
+ * starting at block number <blocks[0]->block_number> and up to
+ * <blocks_length> blocks.  The <block->security_status> is set with
+ * the security status of each block. It is the responsibility of the
+ * caller to set <block->data> to a pointer with enough room to store
+ * the content of a block.
+ */
 int iso15693_read_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* blocks, int blocks_length);
 
 /*
@@ -749,6 +828,18 @@ int iso15693_read_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_t
  */
 #define ISO15693_COMMAND_WRITE_MULTIPLE_BLOCKS			0x24
 
+/*
+ * The WRITE_MULTIPLE_BLOCK is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. Meta information of <transponder>
+ * must be set (using GET_SYSTEM_INFORMATION for instance) before
+ * calling this function. On success, <transponder->bytes_per_block> bytes
+ * are copied from each <block[i]->data> to the corresponding block on
+ * the transponder. This command writes consecutive blocks only. The first
+ * block is <blocks[0]->block_number>. The next block written will be 
+ * <blocks[0]->block_number> + 1 and so on up to <blocks[0]->block_number> + 
+ * <blocks_length>. The <blocks[i > 0]->block_number> fields are ignored.
+ */
 int iso15693_write_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_t* transponder, rfid_block_t* blocks, int blocks_length);
 
 /*
@@ -781,6 +872,10 @@ int iso15693_write_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_
  */
 #define ISO15693_COMMAND_SELECT					0x25
 
+/*
+ * The SELECT command is sent to the transponder with 
+ * UID equal to <uid> (ISO15693_UID_SIZE bytes).
+ */
 int iso15693_select(iso15693_t* iso15693, const u_int8_t* uid);
 
 /*
@@ -806,6 +901,12 @@ int iso15693_select(iso15693_t* iso15693, const u_int8_t* uid);
  */
 #define ISO15693_COMMAND_RESET_TO_READY				0x26
 
+/*
+ * The RESET_TO_READY command is sent to the transponder with 
+ * UID equal to <uid> (ISO15693_UID_SIZE bytes). If <uid> is all
+ * 0, the RESET_TO_READY command is sent to whatever transponder
+ * is available.
+ */
 int iso15693_reset_to_ready(iso15693_t* iso15693, const u_int8_t* uid);
 
 /*
@@ -835,6 +936,14 @@ int iso15693_reset_to_ready(iso15693_t* iso15693, const u_int8_t* uid);
  */
 #define ISO15693_COMMAND_WRITE_AFI				0x27
 
+/*
+ * The WRITE_AFI is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. On success the AFI byte of
+ * <transponder> is written to the transponder. The function
+ * iso15693_transponder_afi_set function must be used by the caller
+ * to set the AFI byte in <transponder>.
+ */
 int iso15693_write_afi(iso15693_t* iso15693, const rfid_transponder_t* transponder);
 
 /*
@@ -863,6 +972,12 @@ int iso15693_write_afi(iso15693_t* iso15693, const rfid_transponder_t* transpond
  */
 #define ISO15693_COMMAND_LOCK_AFI				0x28
 
+/*
+ * The LOCK_AFI command is sent to the transponder with 
+ * UID equal to <uid> (ISO15693_UID_SIZE bytes). If <uid> is all
+ * 0, the LOCK_AFI command is sent to whatever transponder
+ * is available. On success the AFI byte becomes readonly.
+ */
 int iso15693_lock_afi(iso15693_t* iso15693, const u_int8_t* uid);
 
 /*
@@ -892,6 +1007,14 @@ int iso15693_lock_afi(iso15693_t* iso15693, const u_int8_t* uid);
  */
 #define ISO15693_COMMAND_WRITE_DSFID				0x29
 
+/*
+ * The WRITE_DSFID is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. On success the DSFID byte of
+ * <transponder> is written to the transponder. The function
+ * iso15693_transponder_dsfid_set function must be used by the caller
+ * to set the DSFID byte in <transponder>.
+ */
 int iso15693_write_dsfid(iso15693_t* iso15693, const rfid_transponder_t* transponder);
 
 /*
@@ -920,6 +1043,12 @@ int iso15693_write_dsfid(iso15693_t* iso15693, const rfid_transponder_t* transpo
  */
 #define ISO15693_COMMAND_LOCK_DSFID				0x2A
 
+/*
+ * The LOCK_DSFID command is sent to the transponder with 
+ * UID equal to <uid> (ISO15693_UID_SIZE bytes). If <uid> is all
+ * 0, the LOCK_DSFID command is sent to whatever transponder
+ * is available. On success the DSFID byte becomes readonly.
+ */
 int iso15693_lock_dsfid(iso15693_t* iso15693, const u_int8_t* uid);
 
 /*
@@ -973,10 +1102,21 @@ int iso15693_lock_dsfid(iso15693_t* iso15693, const u_int8_t* uid);
  */
 #define ISO15693_COMMAND_GET_SYSTEM_INFORMATION			0x2B
 
+/*
+ * The GET_SYSTEM_INFORMATION is sent to the first available transponder
+ * if the UID is not set in <transponder> (iso15693_transponder_null)
+ * or to a designated transponder. On success the meta information of
+ * the transponder is written in <transponder>. The meta information
+ * fields must be accessed using the 
+ * iso15693_transponder_{flags,afi,dsfid,vicc,ic_ref}* functions declared
+ * in this header. The meaning of the flags must be interpreted according to
+ * the ISO15693_INFO_FLAGS_* macros defined in this header: they define
+ * which field is set and which field is undefined.
+ */
 int iso15693_get_system_information(iso15693_t* iso15693, rfid_transponder_t* transponder);
 
 /*
- * BLOCK_SECURITY_STATUS:
+ * GET_MULTIPLE_BLOCK_SECURITY_STATUS:
  *
  * When receiving the Get multiple block security status command, the
  * VICC shall send back the block security status. The blocks are
@@ -1003,6 +1143,7 @@ int iso15693_get_system_information(iso15693_t* iso15693, rfid_transponder_t* tr
  *   \------------ repeat Number of blocks times ------------/
  */
 #define ISO15693_COMMAND_GET_MULTIPLE_BLOCK_SECURITY_STATUS	0x2C
+/* NOT IMPLEMENTED. */
 
 /*
  * Allocate a reader object that must be free'd by the
@@ -1011,49 +1152,128 @@ int iso15693_get_system_information(iso15693_t* iso15693, rfid_transponder_t* tr
 int iso15693_reader_alloc(rfid_reader_t** reader);
 
 /*
- * Support functions.
+ * Get access to iso15693 object.
+ */
+#define reader_to_iso15693(reader) ((iso15693_t*)reader->private)
+
+/*
+ * Set CRC16 at the end of a ISO-15693 frame (last two bytes).
  */
 void iso15693_crc16_set(u_int8_t* request, int request_length);
+/*
+ * Return true if the CRC16 at the end of a ISO-15693 frame is
+ * correct. Return false otherwise.
+ */
 int iso15693_crc16_check(u_int8_t* request, int request_length);
-int iso15693_transponder_cmp(iso15693_transponder_t* a, iso15693_transponder_t* b);
-int iso15693_transponder_null(iso15693_transponder_t* transponder);
 
+/*
+ * Copy the info flags (see ISO15693_INFO_FLAGS_* macros) from v (u_int8_t) to
+ * transponder t.
+ */
+#define iso15693_transponder_info_flags_set(t,v) \
+        (((iso15693_transponder_t*)(t)->private)->info_flags = (v))
+/*
+ * Copy the info flags (see ISO15693_INFO_FLAGS_* macros) from the transponder t
+ * to v (u_int8_t).
+ */
+#define iso15693_transponder_info_flags_get(t,v) \
+	((v) = ((iso15693_transponder_t*)(t)->private)->info_flags)
+
+/*
+ * Copy the UID from the v memory area (ISO15693_UID_SIZE bytes) to transponder t.
+ */
 #define iso15693_transponder_uid_set(t,v) \
 	memcpy(((iso15693_transponder_t*)(t)->private)->uid, (v), ISO15693_UID_SIZE)
+/*
+ * Copy the UID of transponder t to the v memory area (ISO15693_UID_SIZE bytes).
+ */
 #define iso15693_transponder_uid_get(t,v) \
 	memcpy((v), ((iso15693_transponder_t*)(t)->private)->uid, ISO15693_UID_SIZE)
 
+/*
+ * Copy the DSFID from the v memory area (ISO15693_DSFID_SIZE bytes) to transponder t.
+ */
 #define iso15693_transponder_dsfid_set(t,v) \
         (((iso15693_transponder_t*)(t)->private)->dsfid = (v))
+/*
+ * Copy the DSFID of transponder t to the v memory area (ISO15693_DSFID_SIZE bytes).
+ */
 #define iso15693_transponder_dsfid_get(t,v) \
 	((v) = ((iso15693_transponder_t*)(t)->private)->dsfid)
 
+/*
+ * Copy the AFI from the v memory area (ISO15693_AFI_SIZE bytes) to transponder t.
+ */
 #define iso15693_transponder_afi_set(t,v) \
         (((iso15693_transponder_t*)(t)->private)->afi = (v))
+/*
+ * Copy the AFI of transponder t to the v memory area (ISO15693_AFI_SIZE bytes).
+ */
 #define iso15693_transponder_afi_get(t,v) \
 	((v) = ((iso15693_transponder_t*)(t)->private)->afi)
 
+/*
+ * VICC memory size information bits:
+ *   [0-7] : (8 bits) Number of blocks
+ *   [8-12] : (5 bits) Block size in bytes
+ *   [13-15] : (3 bits) Reserved for future use
+ */
+/*
+ * Copy the VICC from the v variable (u_int16_t) to the transponder t.
+ */
 #define iso15693_transponder_vicc_memory_set(t,v) \
         (((iso15693_transponder_t*)(t)->private)->vicc_memory = (v))
+/*
+ * Copy the VICC of transponder t to the v variable (u_int16_t).
+ */
 #define iso15693_transponder_vicc_memory_get(t,v) \
 	((v) = ((iso15693_transponder_t*)(t)->private)->vicc_memory)
 
+/*
+ * Set to 0 the blocks part of the VICC of the transponder t.
+ */
 #define iso15693_transponder_vicc_blocks_clear(t) \
         (((iso15693_transponder_t*)(t)->private)->vicc_memory &= 0xFF00)
+/*
+ * Copy the v variable (u_int8_t) to the transponder
+ * blocks part of the VICC of the transponder t.
+ */
 #define iso15693_transponder_vicc_blocks_set(t,v) \
         (iso15693_transponder_vicc_blocks_clear(t), (((iso15693_transponder_t*)(t)->private)->vicc_memory |= ((v) & 0xFF)))
+/*
+ * Copy the blocks part of the VICC of the transponder to the v variable (u_int8_t).
+ */
 #define iso15693_transponder_vicc_blocks_get(t,v) \
 	((v) = (((iso15693_transponder_t*)(t)->private)->vicc_memory & 0xFF))
 
+/*
+ * Set to 0 the bytes_per_block part of the VICC of the transponder t.
+ */
 #define iso15693_transponder_vicc_bytes_per_block_clear(t) \
         (((iso15693_transponder_t*)(t)->private)->vicc_memory &= 0xE0FF)
+/*
+ * Copy the v variable (u_int8_t) to the transponder
+ * bytes_per_block part of the VICC of the transponder t.
+ */
 #define iso15693_transponder_vicc_bytes_per_block_set(t,v) \
         (iso15693_transponder_vicc_bytes_per_block_clear(t), (((iso15693_transponder_t*)(t)->private)->vicc_memory |= (((v) & 0x1F) << 8)))
+/*
+ * Copy the bytes_per_block part of the VICC of the transponder
+ * to the v variable (u_int8_t).
+ */
 #define iso15693_transponder_vicc_bytes_per_block_get(t,v) \
 	((v) = ((((iso15693_transponder_t*)(t)->private)->vicc_memory >> 8) & 0x1F))
 
+/*
+ * Copy the IC REFERENCE from the v memory area
+ * (ISO15693_IC_REFERENCE_SIZE bytes) to transponder t.
+ */
 #define iso15693_transponder_ic_ref_set(t,v) \
         (((iso15693_transponder_t*)(t)->private)->ic_ref = (v))
+/*
+ * Copy the IC REFERENCE of transponder t to the v memory area
+ * (ISO15693_IC_REFERENCE_SIZE bytes).
+ */
 #define iso15693_transponder_ic_ref_get(t,v) \
 	((v) = ((iso15693_transponder_t*)(t)->private)->ic_ref)
 
