@@ -105,8 +105,6 @@ int rfid_init(rfid_t* rfid)
   if(reader->init_f(reader) < 0)
     goto finish;
 
-  rfid->last = rfid_transponder_alloc(rfid);
-
   retval = 0;
 
  finish:
@@ -140,7 +138,7 @@ char* rfid_transponder_describe(rfid_t* rfid, rfid_transponder_t* transponder)
   return reader->transponder_describe_f(reader, transponder);
 }
 
-int rfid_transponder_id_get(rfid_t* rfid, rfid_transponder_t* transponder, char** idp)
+int rfid_transponder_id_get(rfid_t* rfid, const rfid_transponder_t* transponder, char** idp)
 {
   rfid_reader_t* reader = rfid->reader;
 
@@ -156,7 +154,6 @@ int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char*
 
 void rfid_end(rfid_t* rfid)
 {
-  rfid_transponder_free(rfid, rfid->last);
   rfid->reader->end_f(rfid->reader);
   rfid->reader->free_f(rfid->reader);
 }
@@ -171,7 +168,9 @@ char* rfid_strerror(rfid_t* rfid)
   if(rfid->error) {
       sprintf(message + strlen(message), "%d: %s", rfid->error, strerror(rfid->error));
   } else {
-    sprintf(message + strlen(message), "%s", reader->strerror_f(reader));
+    char* error_string = reader->strerror_f(reader);
+    sprintf(message + strlen(message), "%s", error_string);
+    free(error_string);
   }
 
   return message;
@@ -251,7 +250,7 @@ typedef struct {
   rfid_transponders_t cache;
 } rfid_events_context_t;
 
-static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context)
+static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int* changedp)
 {
   rfid_transponders_t* current = &context->current;
   rfid_transponders_t* cache = &context->cache;
@@ -294,6 +293,7 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context)
     }
 
     if(compare < 0) {
+      (*changedp)++;
       if(rfid->verbose == 1) fprintf(stderr, "N");
       if(block_signals(rfid) < 0)
 	return RFID_CALLBACK_ERROR;
@@ -331,6 +331,7 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context)
       }
       current_index++;
     } else if(compare > 0) {
+      (*changedp)++;
       if(rfid->verbose == 1) fprintf(stderr, "D");
       if(events_mask & RFID_EVENT_DELETE) {
 	if((retval = (*transponder_callback)(rfid, RFID_EVENT_DELETE, cache->transponders[cache_index], data)) != 0) 
@@ -388,6 +389,8 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
   rfid->error = 0;
 
   while(retval == RFID_CALLBACK_OK) {
+    int changed = 0;
+
     if(block_signals(rfid) < 0)
       return -1;
 
@@ -404,9 +407,10 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
 	continue;
       }
       /*
-       * Ignore recoverable errors.
+       * Restart if a recoverable error occurs.
        */
       if(inventory_error == EAGAIN) {
+	if(rfid->verbose == 1) fprintf(stderr, "!");
 	continue;
       }
 
@@ -418,28 +422,31 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
 
     transponders_sort(rfid, current->transponders, current->length);
 
-    if((retval = transponders_diff(rfid, context)) < 0) {
+    if((retval = transponders_diff(rfid, context, &changed)) < 0) {
       /*
        * Recoverable error (typically the state changed in the middle of
        * the analysis of the current state).
        */
-      if(rfid->error == EAGAIN) {
+      if(rfid_error(rfid) == EAGAIN) {
+	if(rfid->verbose == 1) fprintf(stderr, "!");
 	rfid->error = 0;
 	retval = RFID_CALLBACK_OK;
 	continue;
       }
       /*
-       * Everything else is a genuine error, either from the rfid
+       * Everything else is a fatal error, either from the rfid
        * library or because a callback returned on error.
        */
       return -1;
     }
 
-    if(events_mask & RFID_EVENT_INVENTORY) {
+    if((retval == RFID_CALLBACK_OK) && changed && (events_mask & RFID_EVENT_INVENTORY)) {
       if((retval = (*inventory_callback)(rfid, current->transponders, current->length, data)) < 0)
 	return -1;
     }
     
+    if(rfid->verbose == 1) fprintf(stderr, "/");
+
     /*
      * Swap cache & current transponders. 
      */
