@@ -343,7 +343,7 @@ static int read_answer(s6350_t* s6350, u_int8_t** bufferp, int* buffer_sizep)
   return 0;
 }
 
-char* s6350_strerror(s6350_t* s6350)
+static char* s6350_strerror(s6350_t* s6350)
 {
   char* error_string = strerror(s6350->error);
   char* message = malloc(strlen(error_string) + 32);
@@ -351,7 +351,7 @@ char* s6350_strerror(s6350_t* s6350)
   return message;
 }
 
-int s6350_error(s6350_t* s6350)
+static int s6350_error(s6350_t* s6350)
 {
   return s6350->error;
 }
@@ -699,7 +699,7 @@ static int calcrate(int baudrate)
  * http://www.ti.com/tiris/docs/manuals/refmanuals/RI-STU-TRDCrefGuide.pdf
  * 1.2 Programming Interface
  */
-int s6350_init(s6350_t* s6350)
+static int s6350_init(s6350_t* s6350)
 {
   clear_error(s6350);
   s6350->iso15693_command_size = 128;
@@ -827,7 +827,7 @@ int s6350_init(s6350_t* s6350)
 /*
  * Return 0 if a and b are equal, return non 0 otherwise.
  */
-int s6350_transponder_cmp(s6350_transponder_t* a, s6350_transponder_t* b)
+static int s6350_transponder_cmp(s6350_transponder_t* a, s6350_transponder_t* b)
 {
   return memcmp(a->transponder_id, b->transponder_id, sizeof(u_int8_t) * S6350_TRANSPONDER_DETAILS_TRANSPONDER_ID_SIZE) ||
     a->manufacturer != b->manufacturer ||
@@ -837,7 +837,7 @@ int s6350_transponder_cmp(s6350_transponder_t* a, s6350_transponder_t* b)
 /*
  * Return 1 if the transponder object is not set, 0 otherwise.
  */
-int s6350_transponder_null(s6350_transponder_t* transponder)
+static int s6350_transponder_null(s6350_transponder_t* transponder)
 {
   return
     transponder->transponder_id[3] == 0 &&
@@ -848,7 +848,7 @@ int s6350_transponder_null(s6350_transponder_t* transponder)
     transponder->version_number == 0;
 }
 
-void s6350_end(s6350_t* s6350)
+static void s6350_end(s6350_t* s6350)
 {
   if(s6350) {
     if(s6350->fd > 0)
@@ -858,11 +858,6 @@ void s6350_end(s6350_t* s6350)
   }
 }
 
-const char* s6350_version(void)
-{
-  return S6350_VERSION;
-}
-
 /*
  * rfid_reader driver wrappers.
  */
@@ -906,43 +901,35 @@ static void drv_verbose(struct rfid_reader* reader, int verbosity)
 static int drv_read(struct rfid_reader* reader, int block_start, int number_of_blocks, rfid_transponder_t* transponder)
 {
   s6350_t* s6350 = (s6350_t*)reader->private;
-  rfid_block_t* blocks;
+  rfid_block_t blocks[number_of_blocks];
   int block_map = 0;
+  u_int8_t* pointer;
   int i;
 
-  if(block_start) {
+  transponder->data_length = 0;
+
+  if(block_start >= transponder->blocks ||
+     block_start + number_of_blocks > transponder->blocks) {
     s6350->error = E2BIG;
     return -1;
   }
 
-  if(number_of_blocks > transponder->blocks) {
-    s6350->error = EBADF;
-    return -1;
-  }
-
-  transponder->data_length = number_of_blocks * transponder->bytes_per_block;
+  transponder->data_length = transponder->blocks * transponder->bytes_per_block;
   if(transponder->data_length > S6350_TRANSPONDER_DATA_SIZE) {
-    transponder->data_length = 0;
     s6350->error = EFBIG;
     return -1;
   }
-  memset(transponder->data, '\0', S6350_TRANSPONDER_DATA_SIZE + 1); /* + 1 for trailing NULL */
+  transponder->data[transponder->data_length] = '\0'; /* for trailing NULL */
 
-  blocks = (rfid_block_t*)malloc(sizeof(rfid_block_t) * number_of_blocks);
-
-  block_map = ((1 << number_of_blocks) - 1) & 0xFF;
+  pointer = transponder->data + block_start * transponder->bytes_per_block;
   
-  for(i = 0; i < number_of_blocks; i++)
-    blocks[i].data = &transponder->data[i * transponder->bytes_per_block];
-  
-  if(s6350_special_read_block(s6350, (s6350_transponder_t*)transponder->private, block_map, blocks) < 0) {
-    free(blocks);
-    return -1;
+  for(i = 0; i < number_of_blocks; i++) {
+    block_map |= 1 << (block_start + i);
+    blocks[i].data = pointer;
+    pointer += transponder->bytes_per_block;
   }
-
-  free(blocks);
   
-  return 0;
+  return s6350_special_read_block(s6350, (s6350_transponder_t*)transponder->private, block_map, blocks);
 }
 
 static int drv_write(struct rfid_reader* reader, int block_start, int number_of_blocks, rfid_transponder_t* transponder)
@@ -980,77 +967,48 @@ static int drv_write(struct rfid_reader* reader, int block_start, int number_of_
   if(s6350->verbose == 1) fprintf(stderr, "\n");
   
   return 0;
-#if 0
-  s6350_t* s6350 = (s6350_t*)reader->private;
-
-  if(block_start) {
-    s6350->error = E2BIG;
-    return -1;
-  }
-
-  if(number_of_blocks > transponder->blocks) {
-    s6350->error = EBADF;
-    return -1;
-  }
-
-  if(transponder->data_length > number_of_blocks * transponder->blocks) {
-    s6350->error = EFBIG;
-    return -1;
-  }
-
-  {
-    int block_number;
-    int retval;
-    int offset;
-    u_int8_t* block_data = (u_int8_t*)malloc(transponder->bytes_per_block * number_of_blocks);
-  
-    memset(block_data, '\0', transponder->bytes_per_block * number_of_blocks);
-    memcpy(block_data, transponder->data, transponder->data_length);
-      
-    for(block_number = 0, offset = 0; block_number < number_of_blocks; block_number++, offset += transponder->bytes_per_block) {
-      rfid_block_t block;
-
-      block.data = block_data + offset;
-      block.block_number = block_number;
-      block.security_status = 0;
-
-      while((retval = s6350_write_block(s6350, (s6350_transponder_t*)transponder->private, &block)) < 0 &&
-	    s6350->error == EAGAIN) {
-	if(s6350->verbose == 1) fprintf(stderr, "!");
-      }
-      if(retval < 0) {
-	free(block_data);
-	return -1;
-      }
-      if(s6350->verbose == 1) fprintf(stderr, "W");
-    }
-      
-    free(block_data);
-    if(s6350->verbose == 1) fprintf(stderr, "\n");
-  }
-  
-  return 0;
-#endif
 }
 
 static  int drv_inventory(struct rfid_reader* reader, rfid_transponder_t*** transpondersp, int* transponders_lengthp, int* transponders_sizep)
 {
+  s6350_t* s6350 = (s6350_t*)reader->private;
+
   rfid_transponder_t* transponder = 0;
   rfid_transponder_t** transponders = 0;
 
-  reader->transponder_alloc_f(reader, &transponder);
-  
-  if(reader->transponder_present_f(reader, transponder) < 0) {
-    reader->transponder_free_f(reader, transponder);
-    return -1;
+  if(*transpondersp != 0) {
+    if(*transponders_sizep < 1) {
+      s6350->error = E2BIG;
+      return -1;
+    }
+
+    transponders = *transpondersp;
+    transponder = transponders[0];
+  } else {
+    reader->transponder_alloc_f(reader, &transponder);
+    transponders = (rfid_transponder_t**)malloc(sizeof(rfid_transponder_t*));
+    transponders[0] = transponder;
+    *transponders_sizep = 1;
   }
 
-  transponders = (rfid_transponder_t**)malloc(sizeof(rfid_transponder_t*));
-  transponders[0] = transponder;
+  if(reader->transponder_present_f(reader, transponder) < 0) {
+    if(*transpondersp == 0) {
+      reader->transponder_free_f(reader, transponder);
+      free(transponders);
+    }
+
+    /*
+     * Inventory is not supposed to return ENOENT if no transponder
+     * is found. It merely returns an empty list.
+     */
+    if(reader->error_f(reader) == ENOENT)
+      return 0;
+    else
+      return -1;
+  }
 
   *transpondersp = transponders;
   *transponders_lengthp = 1;
-  *transponders_sizep = 1;
 
   return 0;
 }
@@ -1171,7 +1129,7 @@ static int emulate_iso15693_get_system_information(struct rfid_reader* reader, u
   int data_length = 0;
 
   rfid_transponder_t* transponder = 0;
-  rfid_reader_t* iso15693_reader = reader->parent;
+  rfid_reader_t* iso15693_reader = reader->iso15693_reader;
   iso15693_t* iso15693 = (iso15693_t*)iso15693_reader->private;
   iso15693_inventory_result_t* inventory_results = 0;
 
@@ -1500,7 +1458,7 @@ static int drv_transponder_copy(struct rfid_reader* reader, rfid_transponder_t*
   return 0;
 }
 
-static int drv_transponder_id_get(struct rfid_reader* reader, rfid_transponder_t* transponder, char** idp)
+static int drv_transponder_id_get(struct rfid_reader* reader, const rfid_transponder_t* transponder, char** idp)
 {
   s6350_transponder_t* s6350_transponder = (s6350_transponder_t*)transponder->private;
 
@@ -1559,6 +1517,7 @@ static int drv_alloc(rfid_reader_t** readerp)
   s6350_t* s6350 = (s6350_t*)malloc(sizeof(s6350_t));
   memset(s6350, '\0', sizeof(s6350_t));
 
+  /* Opaque reader information. */
   reader->private = s6350;
 
   reader->alloc_f = drv_alloc;
