@@ -25,14 +25,13 @@
  */
 
 /*
- * For a detailed description of the interface see s6350(3)
+ * For a detailed description see s6350(3)
  */
 
 #ifndef _S6350_H_
 #define _S6350_H_
 
 #include <sys/types.h>
-#include <errno.h>
 
 #include <rfid_reader.h>
 
@@ -46,9 +45,11 @@ typedef struct {
 				   s6350(3) for meaning. */
   char* device;			/* Path name of the device used to
 				   acquire fd. */
-  u_int8_t* iso15693_command;    /* Last ISO-15693 command sent. */
-  int iso15693_command_length;    /* Length of command stored in iso15693_command_length. */
-  int iso15693_command_size;    /* Total number of allocated bytes. */
+  u_int8_t* iso15693_command;   /* Last ISO-15693 command sent. */
+  int iso15693_command_length;  /* Length of command stored in
+				   iso15693_command. */
+  int iso15693_command_size;    /* Total number of bytes allocated
+				   for iso15693_command. */
   int verbose;
 } s6350_t;
 
@@ -89,17 +90,6 @@ typedef struct {
 
 int s6350_reader_alloc(rfid_reader_t** reader);
 
-/*
- * Opens s6350->device that must be a tty and initializes it in raw
- * mode. Probe the presence of a RFID reader and sets the carrier on.
- */
-int s6350_init(s6350_t* s6350);
-
-/*
- * Closes the s6350->fd file descriptor.
- */
-void s6350_end(s6350_t* s6350);
-
 /*
  * Returns the RFID transponder description in <transponder>, if one
  * is present.
@@ -107,24 +97,27 @@ void s6350_end(s6350_t* s6350);
 int s6350_transponder_details(s6350_t* s6350, s6350_transponder_t* transponder);
 
 /*
- * Read RFID transponder blocks according to the <block_numbers> bitmap. Each is
- * stored in the correponding blocks[block_number]. The <transponder> structure is 
- * used to make sure the blocks read match the <transponder> identity. If it is
- * not the case, the read fails.
+ * Read RFID transponder blocks according to the <block_numbers>
+ * bitmap. Each is stored in the correponding
+ * blocks[block_number]. The <transponder> structure is used to make
+ * sure the blocks read match the <transponder> identity. If it is not
+ * the case, the read fails.
  */
 int s6350_special_read_block(s6350_t* s6350, const s6350_transponder_t* transponder, u_int8_t block_numbers, rfid_block_t* blocks);
 
 /*
- * Read a single <block> from the <transponder>. The block->block_number specifies
- * the block number to read. The <transponder> is used to figure out the size
- * of the data (from transponder->bytes_per_block).
+ * Read a single <block> from the <transponder>. The
+ * block->block_number specifies the block number to read. The
+ * <transponder> is used to figure out the size of the data (from
+ * transponder->bytes_per_block).
  */
 int s6350_read_block(s6350_t* s6350, const s6350_transponder_t* transponder, rfid_block_t* block);
 
 /*
- * Write a single <block> to the <transponder>. The block->block_number specifies
- * the block number to read. The <transponder> is used to figure out the size
- * of the data (from transponder->bytes_per_block).
+ * Write a single <block> to the <transponder>. The
+ * block->block_number specifies the block number to read. The
+ * <transponder> is used to figure out the size of the data (from
+ * transponder->bytes_per_block).
  */
 int s6350_write_block(s6350_t* s6350, const s6350_transponder_t* transponder, const rfid_block_t* block);
 
@@ -138,44 +131,4 @@ int s6350_reader_version(s6350_t* s6350, s6350_reader_version_t* reader_version)
  */
 int s6350_rf_carrier(s6350_t* s6350, u_int8_t on_off);
 
-/*
- * Send a ISO-15693 command.
- */
-int s6350_iso15693_command(s6350_t* s6350, u_int8_t* data, int data_length);
-/*
- * Read the response to a ISO-15693 command.
- */
-int s6350_iso15693_response(s6350_t* s6350, u_int8_t** bufferp, int* buffer_sizep);
-
-/*
- * Error codes from the RFID reader.
- */
-#define S6350_ERROR_TRANSPONDER_NOT_FOUND			ENOENT
-#define S6350_ERROR_COMMAND_NOT_SUPPORTED			EPERM
-#define S6350_ERROR_PACKET_BCC_INVALID				EPIPE
-#define S6350_ERROR_PACKET_FLAGS_INVALID_FOR_COMMAND		ERANGE
-#define S6350_ERROR_GENERAL_WRITE_FAILURE			ENXIO
-#define S6350_ERROR_WRITE_FAILURE_DUE_TO_LOCK_BLOCK		ENOLCK
-#define S6350_ERROR_TRANSPONDER_DOES_NOT_SUPPORT_FUNCTION	ENOTSUP
-#define S6350_ERROR_UNDEFINED_ERROR				ESRCH
-#define S6350_ERROR_INVALID					ENOSYS
-
-int s6350_transponder_cmp(s6350_transponder_t* a, s6350_transponder_t* b);
-int s6350_transponder_null(s6350_transponder_t* transponder);
-
-/*
- * Convert RFID reader error codes into strings.
- */
-char* s6350_strerror(s6350_t* s6350);
-/*
- * Return the latest error number. Positive values are errno(3)
- * values, negative values are S6350 reader specific error numbers.
- */
-int s6350_error(s6350_t* s6350);
-
-/*
- * Return the version of the library as a string (1.0.0 for instance).
- */
-const char* s6350_version(void);
-
 #endif /* _S6350_H_ */
