@@ -36,13 +36,15 @@
 #include <stdio.h>
 #include <signal.h>
 
-static void report_error(s6350_t* s6350, const char* message)
+static void report_error(rfid_reader_t* reader, const char* message)
 {
-  fprintf(stderr, "Device %s\n", s6350->device);
-  fprintf(stderr, "%s: %s\n", message, s6350_strerror(s6350));
+  char* str = reader->strerror_f(reader);
+  fprintf(stderr, "%s: %s\n", message, str);
+  free(str);
 }
 
-static s6350_t s6350;
+static s6350_t* s6350;
+static rfid_reader_t* reader;
 
 static RETSIGTYPE alarm_f(int sig)
 {
@@ -51,26 +53,31 @@ static RETSIGTYPE alarm_f(int sig)
 FAILURE: Timeout attempting to establish communication with RFID reader at %s.\n\
 FAILURE: Make sure a RFID reader is plugged in and turned on.\n\
 \n\
-", s6350.device);
+", reader->device);
   exit(-1);
 }
 
 int main(int argc, char** argv)
 {
+  int timeout = 5;
+
+  if(s6350_reader_alloc(&reader) < 0)
+    return -1;
+  s6350 = (s6350_t*)reader->private;
 
   if(argc > 1)
-    s6350.device = argv[1];
+    reader->device = argv[1];
   else
-    s6350.device = "/dev/ttyS0";
+    reader->device = "/dev/ttyS0";
 
   if(signal(SIGALRM, alarm_f) < 0) {
     perror("signal(SIGALRM)");
     return -1;
   }
 
-  alarm(5);
-  if(s6350_init(&s6350) < 0) {
-    report_error(&s6350, "s6350_init");
+  alarm(timeout);
+  if(reader->init_f(reader) < 0) {
+    report_error(reader, "init_f");
   }
   alarm(0);
 
@@ -78,8 +85,8 @@ int main(int argc, char** argv)
     s6350_transponder_t transponder;
     rfid_block_t block = { 0 };
    
-    if(s6350_transponder_details(&s6350, &transponder)) {
-      if(s6350.error == ENOENT) {
+    if(s6350_transponder_details(s6350, &transponder)) {
+      if(reader->error_f(reader) == ENOENT) {
 	fprintf(stderr, "\
 \n\
 FAILURE: The test program will attempt to read / write on a transponder,\n\
@@ -87,7 +94,7 @@ FAILURE: please put a transponder on the RFID reader.\n\
 \n\
 ");
       }
-      report_error(&s6350, "s6350_transponder_details");
+      report_error(reader, "s6350_transponder_details");
       return -1;
     }
 
@@ -96,15 +103,15 @@ FAILURE: please put a transponder on the RFID reader.\n\
     block.data[0] = block.data[1] = block.data[2] = block.data[3] = 'B';
     block.block_number = 0;
 
-    if(s6350_write_block(&s6350, &transponder, &block)) {
-      report_error(&s6350, "s6350_write_block");
+    if(s6350_write_block(s6350, &transponder, &block)) {
+      report_error(reader, "s6350_write_block");
       return -1;
     }
 
     for(block.block_number = 0; block.block_number < transponder.blocks; block.block_number++) {
 	
-      if(s6350_read_block(&s6350, &transponder, &block)) {
-	report_error(&s6350, "s6350_read_block");
+      if(s6350_read_block(s6350, &transponder, &block)) {
+	report_error(reader, "s6350_read_block");
 	return -1;
       }
     }
@@ -118,8 +125,8 @@ FAILURE: please put a transponder on the RFID reader.\n\
       for(i = 0; i < 32; i++)
 	blocks[i].data = (u_int8_t*)malloc(transponder.bytes_per_block);
 
-      if(s6350_special_read_block(&s6350, &transponder, 0xFF, blocks)) {
-	report_error(&s6350, "s6350_special_read_block");
+      if(s6350_special_read_block(s6350, &transponder, 0xFF, blocks)) {
+	report_error(reader, "s6350_special_read_block");
 	return -1;
       }
 
@@ -130,7 +137,8 @@ FAILURE: please put a transponder on the RFID reader.\n\
 
   }
   
-  s6350_end(&s6350);
+  reader->end_f(reader);
+  reader->free_f(reader);
   
   return 0;
 }
